@@ -57,6 +57,16 @@ else:
         del encoding
     except LookupError:
         pass
+    else:
+        def raw_input(prompt="", _raw_input=raw_input):
+            if prompt:
+                prompt = sys.stdout.encode(prompt)[0]
+            sys_stdout = sys.stdout
+            sys.stdout = sys.__stdout__
+            res = _raw_input(prompt)
+            sys.stdout = sys_stdout
+            return res
+        __builtins__["raw_input"] = raw_input
 
 class Proxy:
     def __init__(self, object=None):
@@ -75,7 +85,8 @@ def init(command=None, argv=None,
          datadir=None, configfile=None,
          gui=False, shell=False, interface=None,
          forcelocks=False, loglevel=None):
-    from smart.const import DEBUG, INFO, WARNING, ERROR, DISTROFILE, DATADIR
+    from smart.const import DEBUG, INFO, WARNING, ERROR
+    from smart.const import DATADIR, USERDATADIR
     from smart.interface import Interface, createInterface
     from smart.sysconfig import SysConfig
     from smart.pkgconfig import PkgConfig
@@ -95,6 +106,7 @@ def init(command=None, argv=None,
         sysconf.set("log-level", level, soft=True)
     if datadir:
         sysconf.set("data-dir", os.path.expanduser(datadir), soft=True)
+    sysconf.set("user-data-dir", os.path.expanduser(USERDATADIR), soft=True)
     ctrl = Control(configfile, forcelocks)
     if gui:
         ifacename = sysconf.get("default-gui", "gtk")
@@ -108,14 +120,17 @@ def init(command=None, argv=None,
         ifacename = "text"
     iface.object = createInterface(ifacename, ctrl, command, argv)
 
+    return ctrl
+
+def initDistro(ctrl):
     # Run distribution script, if available.
-    if os.path.isfile(DISTROFILE):
-        execfile(DISTROFILE, {"ctrl": ctrl, "iface": iface,
+    from smart.const import DISTROFILE
+    distrofile = sysconf.get("distro-init-file", DISTROFILE)
+    if distrofile and os.path.isfile(distrofile):
+        execfile(distrofile, {"ctrl": ctrl, "iface": iface,
                               "sysconf": sysconf, "pkgconf": pkgconf,
                               "hooks": hooks})
 
-    return ctrl
-
 def initPlugins():
     # Import every plugin, and let they do whatever they want. Backends
     # are also considered plugins for that matter.
@@ -143,4 +158,16 @@ def initPlugins():
             if os.path.isfile(initpath):
                 __import__("smart.backends."+entry)
 
+def initPsyco():
+    if sysconf.get("psyco", True):
+        try:
+            import psyco
+        except ImportError:
+            pass
+        else:
+            hooks.call("enable-psyco", psyco)
+            def autocall(hookname, hookfunc, priority, threaded, psyco=psyco):
+                hookfunc(psyco)
+            hooks.register("enable-psyco-registered", autocall)
+
 # vim:ts=4:sw=4:et
