@@ -24,99 +24,51 @@
 # being linked with rpm. :-(
 import zlib
 
-from smart.backends.rpm.pm import RPMPackageManager
-from rpmver import checkdep, vercmp, splitarch
+from rpmver import checkdep, vercmp, splitarch, splitrelease
 from smart.util.strtools import isGlob
-from smart.matcher import Matcher
 from smart.cache import *
+from smart import *
 import fnmatch
 import string
 import os, re
 
-from rpm import archscore
+try:
+    import rpm
+except ImportError:
+    raise Error, _("'rpm' python module is not available")
+
+archscore = rpm.archscore
 
 __all__ = ["RPMPackage", "RPMProvides", "RPMNameProvides", "RPMPreRequires",
-           "RPMRequires", "RPMUpgrades", "RPMConflicts", "RPMObsoletes"]
+           "RPMRequires", "RPMUpgrades", "RPMConflicts", "RPMObsoletes",
+           "rpm", "getTS"]
 
-class RPMMatcher(Matcher):
-
-    def __init__(self, str):
-        Matcher.__init__(self, str)
-        self._options = [] # (name, version)
-        # First, try to match the whole thing against the name.
-        name = str
-        if isGlob(name):
+def getTS(new=False):
+    if not hasattr(getTS, "ts"):
+        getTS.root = sysconf.get("rpm-root", "/")
+        getTS.ts = rpm.ts(getTS.root)
+        if not os.path.isfile(os.path.join(getTS.root, "var/lib/rpm/Packages")):
             try:
-                name = re.compile(fnmatch.translate(name))
-            except re.error:
-                pass
-        self._options.append((name, None))
-        tokens = str.split("-")
-        if len(tokens) > 1:
-            # Then, consider the last section as the version.
-            name = "-".join(tokens[:-1])
-            if isGlob(name):
-                try:
-                    name = re.compile(fnmatch.translate(name))
-                except re.error:
-                    pass
-            version = tokens[-1]
-            if isGlob(version):
-                if ":" not in version and version[0].isdigit():
-                    pattern = "(?:\d+:)?"+fnmatch.translate(version)
-                else:
-                    pattern = version
-                try:
-                    version = re.compile(pattern)
-                except re.error:
-                    pass
-            self._options.append((name, version))
-            # Finally, consider last two sections as the version.
-            if len(tokens) > 2:
-                name = "-".join(tokens[:-2])
-                if isGlob(name):
-                    try:
-                        name = re.compile(fnmatch.translate(name))
-                    except re.error:
-                        pass
-                version = "-".join(tokens[-2:])
-                if isGlob(version):
-                    if ":" not in version and version[0].isdigit():
-                        pattern = "(?:\d+:)?"+fnmatch.translate(version)
-                    else:
-                        pattern = version
-                    try:
-                        version = re.compile(pattern)
-                    except re.error:
-                        pass
-                self._options.append((name, version))
-
-    def matches(self, obj):
-        for name, version in self._options:
-            if type(name) is str:
-                if name != obj.name:
-                    continue
+                getTS.ts.initDB()
+            except rpm.error:
+                raise Error, _("Couldn't initizalize rpm database at %s") \
+                             % getTS.root
             else:
-                if not name.match(obj.name):
-                    continue
-            if version:
-                if type(version) is str:
-                    if ":" not in version and ":" in obj.version:
-                        ov = obj.version
-                        version = ov[:ov.find(":")+1]+version
-                    objver, objarch = splitarch(obj.version)
-                    if vercmp(version, objver) != 0:
-                        ver, arch = splitarch(version)
-                        if arch != objarch or vercmp(ver, objver) != 0:
-                            continue
-                elif not version.match(obj.version):
-                    continue
-            return True
+                iface.warning(_("Initialized new rpm database at %s")
+                              % getTS.root)
+    if new:
+        return rpm.ts(getTS.root)
+    else:
+        return getTS.ts
+
+# Here because pm requires getTS and rpm defined/imported above.
+from smart.backends.rpm.pm import RPMPackageManager
 
 class RPMPackage(Package):
 
+    __slots__ = ()
+
     packagemanager = RPMPackageManager
-    matcher = RPMMatcher
 
     def equals(self, other):
         if self.name != other.name or self.version != other.version:
@@ -176,6 +128,34 @@ class RPMPackage(Package):
         ver, arch = splitarch(version)
         return checkdep(selfver, relation, ver)
 
+    def search(self, searcher, _epochre=re.compile("[0-9]+:")):
+        myname = self.name
+        myversion, myarch = splitarch(self.version)
+        myversion = _epochre.sub("", myversion)
+        ratio = 0
+        ic = searcher.ignorecase
+        for nameversion, cutoff in searcher.nameversion:
+            nameversion = _epochre.sub("", nameversion)
+            if '@' in nameversion:
+                _, ratio1 = globdistance(nameversion, "%s-%s" %
+                                         (myname, self.version), cutoff, ic)
+                _, ratio2 = globdistance(nameversion, "%s@%s" %
+                                         (myname, myarch), cutoff, ic)
+                _, ratio3 = globdistance(nameversion, "%s-%s@%s" %
+                                         (myname, splitrelease(myversion)[0],
+                                          myarch), cutoff, ic)
+            else:
+                _, ratio1 = globdistance(nameversion, myname, cutoff, ic)
+                _, ratio2 = globdistance(nameversion,
+                                         "%s-%s" % (myname, myversion),
+                                         cutoff, ic)
+                _, ratio3 = globdistance(nameversion, "%s-%s" %
+                                         (myname, splitrelease(myversion)[0]),
+                                         cutoff, ic)
+            ratio = max(ratio, ratio1, ratio2, ratio3)
+        if ratio:
+            searcher.addResult(self, ratio)
+
     def __lt__(self, other):
         rc = cmp(self.name, other.name)
         if type(other) is RPMPackage:
@@ -188,11 +168,13 @@ class RPMPackage(Package):
                     rc = -cmp(archscore(selfarch), archscore(otherarch))
         return rc == -1
 
-class RPMProvides(Provides): pass
-class RPMNameProvides(RPMProvides): pass
+class RPMProvides(Provides):         __slots__ = ()
+class RPMNameProvides(RPMProvides):  __slots__ = ()
 
 class RPMDepends(Depends):
 
+    __slots__ = ()
+
     def matches(self, prv):
         if not isinstance(prv, RPMProvides) and type(prv) is not Provides:
             return False
@@ -202,12 +184,13 @@ class RPMDepends(Depends):
         prvver, prvarch = splitarch(prv.version)
         return checkdep(prvver, self.relation, selfver)
 
-class RPMPreRequires(RPMDepends,PreRequires): pass
-class RPMRequires(RPMDepends,Requires): pass
-class RPMUpgrades(RPMDepends,Upgrades): pass
-class RPMConflicts(RPMDepends,Conflicts): pass
+class RPMPreRequires(RPMDepends,PreRequires): __slots__ = ()
+class RPMRequires(RPMDepends,Requires):       __slots__ = ()
+class RPMUpgrades(RPMDepends,Upgrades):       __slots__ = ()
+class RPMConflicts(RPMDepends,Conflicts):     __slots__ = ()
 
 class RPMObsoletes(Depends):
+    __slots__ = ()
 
     def matches(self, prv):
         if not isinstance(prv, RPMNameProvides) and type(prv) is not Provides:
@@ -228,4 +211,15 @@ _COLORMAP = {"x86_64": 2, "ppc64": 2, "s390x": 2, "sparc64": 2}
 def getArchColor(arch, _cm=_COLORMAP):
     return _cm.get(arch, 1)
 
+def enablePsyco(psyco):
+    psyco.bind(RPMPackage.equals)
+    psyco.bind(RPMPackage.coexists)
+    psyco.bind(RPMPackage.matches)
+    psyco.bind(RPMPackage.search)
+    psyco.bind(RPMPackage.__lt__)
+    psyco.bind(RPMDepends.matches)
+    psyco.bind(RPMObsoletes.matches)
+
+hooks.register("enable-psyco", enablePsyco)
+
 # vim:ts=4:sw=4:et
