@@ -20,15 +20,21 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 from smart.backends.rpm.rpmver import splitarch
+from smart.util.strtools import globdistance
 from smart.cache import Loader, PackageInfo
 from smart.channel import FileChannel
 from smart.backends.rpm.base import *
+from smart.progress import Progress
 from smart import *
 import locale
 import stat
-import rpm
 import os
 
+try:
+    import rpmhelper
+except ImportError:
+    rpmhelper = None
+
 CRPMTAG_FILENAME          = 1000000
 CRPMTAG_FILESIZE          = 1000001
 CRPMTAG_MD5               = 1000005
@@ -278,6 +284,69 @@ class RPMHeaderLoader(Loader):
             self._offsets[offset] = pkg
             self._groups[pkg] = intern(h[rpm.RPMTAG_GROUP])
 
+    def search(self, searcher):
+        ic = searcher.ignorecase
+        for h, offset in self.getHeaders(Progress()):
+            pkg = self._offsets.get(offset)
+            if not pkg:
+                continue
+
+            ratio = 0
+            if searcher.url:
+                refurl = h[rpm.RPMTAG_URL]
+                if refurl:
+                    for url, cutoff in searcher.url:
+                        _, newratio = globdistance(url, refurl, cutoff, ic)
+                        if newratio > ratio:
+                            ratio = newratio
+                            if ratio == 1:
+                                break
+            if ratio == 1:
+                searcher.addResult(pkg, ratio)
+                continue
+            if searcher.path:
+                paths = h[rpm.RPMTAG_OLDFILENAMES]
+                if paths:
+                    for spath, cutoff in searcher.path:
+                        for path in paths:
+                            _, newratio = globdistance(spath, path, cutoff, ic)
+                            if newratio > ratio:
+                                ratio = newratio
+                                if ratio == 1:
+                                    break
+                        else:
+                            continue
+                        break
+            if ratio == 1:
+                searcher.addResult(pkg, ratio)
+                continue
+            if searcher.group:
+                group = self._groups[pkg]
+                for pat in searcher.group:
+                    if pat.search(group):
+                        ratio = 1
+                        break
+            if ratio == 1:
+                searcher.addResult(pkg, ratio)
+                continue
+            if searcher.summary:
+                summary = h[rpm.RPMTAG_SUMMARY]
+                for pat in searcher.summary:
+                    if pat.search(summary):
+                        ratio = 1
+                        break
+            if ratio == 1:
+                searcher.addResult(pkg, ratio)
+                continue
+            if searcher.description:
+                description = h[rpm.RPMTAG_DESCRIPTION]
+                for pat in searcher.description:
+                    if pat.search(description):
+                        ratio = 1
+                        break
+            if ratio:
+                searcher.addResult(pkg, ratio)
+
 class RPMHeaderListLoader(RPMHeaderLoader):
 
     def __init__(self, filename, baseurl, count=None):
@@ -453,23 +522,31 @@ class RPMDBLoader(RPMHeaderLoader):
         self._infoorder = -100
 
     def getLoadSteps(self):
-        return 1
+        # Estimate, since there's no other good way to do it.
+        return 1000
 
     def getHeaders(self, prog):
-        ts = rpm.ts(sysconf.get("rpm-root", "/"))
-        mi = ts.dbMatch()
+        mi = getTS().dbMatch()
+        total = left = self.getLoadSteps()
         for h in mi:
             if h[1000] != "gpg-pubkey": # RPMTAG_NAME
                 yield h, mi.instance()
-            prog.addTotal(1)
+            if left == 0:
+                prog.addTotal(1)
+            else:
+                left -= 1
             prog.add(1)
             prog.show()
-        prog.add(1)
+        prog.add(left)
 
-    def getHeader(self, pkg):
-        ts = rpm.ts(sysconf.get("rpm-root", "/"))
-        mi = ts.dbMatch(0, pkg.loaders[self])
-        return mi.next()
+    if rpmhelper:
+        def getHeader(self, pkg):
+            mi = rpmhelper.dbMatch(getTS(), 0, pkg.loaders[self])
+            return mi.next()
+    else:
+        def getHeader(self, pkg):
+            mi = getTS().dbMatch(0, pkg.loaders[self])
+            return mi.next()
 
     def getURL(self):
         return None
@@ -484,7 +561,7 @@ class RPMDBLoader(RPMHeaderLoader):
         return None
 
     def loadFileProvides(self, fndict):
-        ts = rpm.ts(sysconf.get("rpm-root", "/"))
+        ts = getTS()
         bfp = self.buildFileProvides
         for fn in fndict:
             mi = ts.dbMatch(1117, fn) # RPMTAG_BASENAMES
@@ -514,7 +591,7 @@ class RPMDirLoader(RPMHeaderLoader):
         return len(self._filenames)
 
     def getHeaders(self, prog):
-        ts = rpm.ts()
+        ts = getTS()
         for i, filename in enumerate(self._filenames):
             filepath = os.path.join(self._dir, filename)
             file = open(filepath)
@@ -532,7 +609,7 @@ class RPMDirLoader(RPMHeaderLoader):
         filename = self._filenames[pkg.loaders[self]]
         filepath = os.path.join(self._dir, filename)
         file = open(filepath)
-        ts = rpm.ts()
+        ts = getTS()
         try:
             h = ts.hdrFromFdno(file.fileno())
         except rpm.error, e:
@@ -562,7 +639,7 @@ class RPMDirLoader(RPMHeaderLoader):
         return None
 
     def loadFileProvides(self, fndict):
-        ts = rpm.ts()
+        ts = getTS()
         bfp = self.buildFileProvides
         for i, filename in enumerate(self._filenames):
             if i not in self._offsets:
@@ -602,4 +679,18 @@ def createFileChannel(filename):
 
 hooks.register("create-file-channel", createFileChannel)
 
+def enablePsyco(psyco):
+    psyco.bind(RPMHeaderLoader.load)
+    psyco.bind(RPMHeaderLoader.search)
+    psyco.bind(RPMHeaderListLoader.getHeaders)
+    psyco.bind(RPMHeaderListLoader.getHeadersHDL)
+    psyco.bind(RPMHeaderListLoader.loadFileProvides)
+    psyco.bind(RPMHeaderListLoader.loadFileProvidesHDL)
+    psyco.bind(RPMDirLoader.getHeaders)
+    psyco.bind(RPMDirLoader.loadFileProvides)
+    psyco.bind(RPMDBLoader.getHeaders)
+    psyco.bind(RPMDBLoader.loadFileProvides)
+
+hooks.register("enable-psyco", enablePsyco)
+
 # vim:ts=4:sw=4:et
