@@ -30,7 +30,24 @@ Repository created for Mandrake's URPMI package manager.
 """)
 
 fields = [("baseurl", _("Base URL"), str, None,
-           _("Base URL where packages are found under.")),
+           _("Base URL where packages or the 'list' file are found under. "
+             "Using ' with <hdlurl>' pattern is also supported.")),
           ("hdlurl", _("Header List URL"), str, "",
            _("URL for header list (hdlist or synthesis). If it's hdlist.cz "
-             "inside the given base URL, may be left empty."))]
+             "inside the given base URL, may be left empty. URLs relative "
+             "to the Base URL are supported"))]
+
+def postParse(data):
+    import re
+    withre = re.compile("\s+with\s+", re.I)
+    if withre.search(data["baseurl"]):
+        if "hdlurl" in data:
+            raise Error, _("Base URL has 'with', but Header List URL "
+                           "was provided")
+        tokens = withre.split(data["baseurl"])
+        if len(tokens) != 2:
+            raise Error, _("Base URL has invalid 'with' pattern")
+        data["baseurl"] = tokens[0].strip()
+        if tokens[1].strip():
+            data["hdlurl"] = tokens[1].strip()
+    return data
