@@ -20,9 +20,8 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 from smart.util.strtools import isGlob
-from smart.matcher import MasterMatcher
 from smart.option import OptionParser
-from smart.cache import Provides, PreRequires
+from smart.cache import Provides, PreRequires, Package
 from smart import *
 import tempfile
 import fnmatch
@@ -108,6 +107,8 @@ def parse_options(argv, help=None):
                              "information"))
     parser.add_option("--show-priority", action="store_true",
                       help=_("show package priority"))
+    parser.add_option("--show-channels", action="store_true",
+                      help=_("show channels that include this package"))
     parser.add_option("--show-all", action="store_true",
                       help=_("enable all --show-* options"))
     parser.add_option("--format", action="store", default="text",
@@ -131,10 +132,29 @@ def main(ctrl, opts, reloadchannels=True):
     if not opts.args:
         packages = cache.getPackages()[:]
     else:
-        packages = []
+        packages = {}
         for arg in opts.args:
-            matcher = MasterMatcher(arg)
-            packages.extend(matcher.filter(cache.getPackages()))
+            ratio, results, suggestions = ctrl.search(arg, addprovides=False)
+            if not results:
+                if suggestions:
+                    dct = {}
+                    for r, obj in suggestions:
+                        if isinstance(obj, Package):
+                            dct[obj] = True
+                        else:
+                            dct.update(dct.fromkeys(obj.packages, True))
+                    raise Error, _("'%s' matches no packages. "
+                                   "Suggestions:\n%s") % \
+                                 (arg, "\n".join(["    "+str(x) for x in dct]))
+                else:
+                    raise Error, _("'%s' matches no packages") % arg
+            else:
+                for obj in results:
+                    if isinstance(obj, Package):
+                        packages[obj] = True
+                    else:
+                        packages.update(dict.fromkeys(obj.packages, True))
+        packages = packages.keys()
 
     if opts.installed:
         packages = [pkg for pkg in packages if pkg.installed]
@@ -326,7 +346,8 @@ def main(ctrl, opts, reloadchannels=True):
                             if opts.installed and not cnfpkg.installed:
                                 continue
                             output.showConflictedBy(pkg, prv, cnf, cnfpkg)
-        if pkg.requires and (opts.show_requires or opts.show_prerequires):
+        if pkg.requires and (opts.show_requires or opts.show_prerequires
+                             or whorequires):
             pkg.requires.sort()
             first = True
             for req in pkg.requires:
@@ -348,7 +369,7 @@ def main(ctrl, opts, reloadchannels=True):
                                 continue
                             output.showRequiresProvidedBy(pkg, req,
                                                           prv, prvpkg)
-        if pkg.upgrades and opts.show_upgrades:
+        if pkg.upgrades and (opts.show_upgrades or whoupgrades):
             pkg.upgrades.sort()
             first = True
             for upg in pkg.upgrades:
@@ -368,7 +389,7 @@ def main(ctrl, opts, reloadchannels=True):
                                 continue
                             output.showUpgradesProvidedBy(pkg, upg,
                                                           prv, prvpkg)
-        if pkg.conflicts and opts.show_conflicts:
+        if pkg.conflicts and (opts.show_conflicts or whoconflicts):
             pkg.conflicts.sort()
             first = True
             for cnf in pkg.conflicts:
@@ -391,10 +412,10 @@ def main(ctrl, opts, reloadchannels=True):
                             output.showConflictsProvidedBy(pkg, upg,
                                                            prv, prvpkg)
 
-    output.stopGrabOutput()
-
     output.end()
 
+    output.stopGrabOutput()
+
 
 class NullOutput(object):
 
@@ -412,8 +433,8 @@ class NullOutput(object):
     def startGrabOutput(self, output=None):
         if output or self.opts.output:
             self.output = output or open(self.opts.output, "w")
-            sys.stdout = self.output
             self.__sys_stdout = sys.stdout
+            sys.stdout = self.output
         else:
             self.output = sys.stdout
             self.__sys_stdout = None
@@ -472,6 +493,12 @@ class TextOutput(NullOutput):
             print pkg,
         if self.opts.show_priority:
             print "{%s}" % pkg.getPriority(),
+        if self.opts.show_channels:
+            channels = []
+            for loader in pkg.loaders:
+                channels.append(loader.getChannel().getAlias())
+            channels.sort()
+            print "[%s]" % ', '.join(channels),
         if self.opts.show_summary:
             info = pkg.loaders.keys()[0].getInfo(pkg)
             print "-", info.getSummary(),
@@ -489,6 +516,9 @@ class TextOutput(NullOutput):
         self._firstconflictsprovidedby = True
 
     def showProvides(self, pkg, prv):
+        self._firstrequiredby = True
+        self._firstupgradedby = True
+        self._firstconflictedby = True
         if self._firstprovides:
             self._firstprovides = False
             print " ", _("Provides:")
@@ -673,10 +703,92 @@ class DottyOutput(GraphVizOutput):
 
     def end(self):
         GraphVizOutput.end(self)
+        self.stopGrabOutput()
         try:
             os.system("dotty %s" % self.__filename)
         finally:
             if self.__filename != self.opts.output:
                 os.unlink(self.__filename)
 
+class PrologOutput(NullOutput):
+
+    def add(self, fact):
+        self._facts[fact] = True
+
+    def start(self):
+        self._facts = {}
+        self._firstrequires = True
+        self._firstrequiredby = True
+
+    def end(self):
+        facts = self._facts.keys()
+        self._facts.clear()
+        facts.sort()
+        for fact in facts:
+            print fact
+        print
+    
+    def showPackage(self, pkg):
+        self.add("package('%s')." % pkg)
+        if self.opts.show_priority:
+            self.add("priority('%s', %d)." % (pkg, pkg.getPriority()))
+
+    def showProvides(self, pkg, prv):
+        self.add("provides('%s', '%s')." % (pkg, prv))
+
+    def showRequiredBy(self, pkg, prv, req, reqpkg):
+        self.showPackage(reqpkg)
+        self.showRequires(reqpkg, req)
+        tup = (prv, req)
+        if isinstance(req, PreRequires):
+            self.add("prerequiredby('%s', '%s')." % (prv, req))
+        else:
+            self.add("requiredby('%s', '%s')." % (prv, req))
+        if self._firstrequiredby:
+            self._firstrequiredby = False
+        self.add("requiredby(X, Y) :- prerequiredby(X, Y).")
+
+    def showUpgradedBy(self, pkg, prv, upg, upgpkg):
+        self.showPackage(upgpkg)
+        self.showUpgrades(upgpkg, upg)
+        self.add("upgradedby('%s', '%s')." % (prv, upg))
+
+    def showConflictedBy(self, pkg, prv, cnf, cnfpkg):
+        self.showPackage(cnfpkg)
+        self.showConflicts(cnfpkg, cnf)
+        self.add("conflictedby('%s', '%s')." % (prv, cnf))
+
+    def showRequires(self, pkg, req):
+        if isinstance(req, PreRequires):
+            self.add("prerequires('%s', '%s')." % (pkg, req))
+        else:
+            self.add("requires('%s', '%s')." % (pkg, req))
+        if self._firstrequires:
+            self._firstrequires = False
+            self.add("requires(X, Y) :- prerequires(X, Y).")
+
+    def showRequiresProvidedBy(self, pkg, req, prv, prvpkg):
+        self.showPackage(prvpkg)
+        self.showProvides(prvpkg, prv)
+        if isinstance(req, PreRequires):
+            self.add("prerequiredby('%s', '%s')." % (prv, req))
+        else:
+            self.add("requiredby('%s', '%s')." % (prv, req))
+
+    def showUpgrades(self, pkg, upg):
+        self.add("upgrades('%s', '%s')." % (pkg, upg))
+
+    def showUpgradesProvidedBy(self, pkg, upg, prv, prvpkg):
+        self.showPackage(prvpkg)
+        self.showProvides(prvpkg, prv)
+        self.add("upgradedby('%s', '%s')." % (prv, upg))
+
+    def showConflicts(self, pkg, cnf):
+        self.add("conflicts('%s', '%s')." % (pkg, cnf))
+
+    def showConflictsProvidedBy(self, pkg, cnf, prv, prvpkg):
+        self.showPackage(prvpkg)
+        self.showProvides(prvpkg, prv)
+        self.add("conflictedby('%s', '%s')." % (prv, cnf))
+
 # vim:ts=4:sw=4:et
