@@ -21,7 +21,7 @@
 #
 import sys
 
-VERSION = "0.29.2"
+VERSION = "0.36"
 
 RECURSIONLIMIT = sys.getrecursionlimit()
 
@@ -52,21 +52,22 @@ NEVER     = Enum("NEVER")
 ENFORCE   = Enum("ENFORCE")
 ALWAYS    = Enum("ALWAYS")
 
-ERROR     = Enum("ERROR")
-WARNING   = Enum("WARNING")
-INFO      = Enum("INFO")
-DEBUG     = Enum("DEBUG")
-
 WAITING   = Enum("WAITING")
 RUNNING   = Enum("RUNNING")
 FAILED    = Enum("FAILED")
 SUCCEEDED = Enum("SUCCEEDED")
 
+ERROR     = 1
+WARNING   = 2
+INFO      = 3
+DEBUG     = 4
+
 BLOCKSIZE = 16384
 
-DISTROFILE = "/usr/lib/smart/distro.py"
-PLUGINSDIR = "/usr/lib/smart/plugins/"
-DATADIR    = "/var/lib/smart/"
-CONFFILE   = "config"
+DISTROFILE  = "/usr/lib/smart/distro.py"
+PLUGINSDIR  = "/usr/lib/smart/plugins/"
+DATADIR     = "/var/lib/smart/"
+USERDATADIR = "~/.smart/"
+CONFFILE    = "config"
 
 # vim:ts=4:sw=4:et
