@@ -24,11 +24,12 @@ from smart.util.filetools import compareFiles, setCloseOnExecAll
 from smart.util.objdigest import getObjectDigest
 from smart.util.pathlocks import PathLocks
 from smart.util.strtools import strToBool
+from smart.searcher import Searcher
 from smart.media import MediaSet
 from smart.progress import Progress
 from smart.fetcher import Fetcher
-from smart.cache import Cache, StateVersionError
 from smart.channel import *
+from smart.cache import *
 from smart.const import *
 from smart import *
 import cPickle
@@ -124,19 +125,16 @@ class Control(object):
     __stateversion__ = 2
 
     def loadSysConf(self, confpath=None):
-        loaded = False
         datadir = sysconf.get("data-dir")
         if confpath:
             confpath = os.path.expanduser(confpath)
             if not os.path.isfile(confpath):
                 raise Error, _("Configuration file not found: %s") % confpath
             sysconf.load(confpath)
-            loaded = True
         else:
             confpath = os.path.join(datadir, CONFFILE)
             if os.path.isfile(confpath):
                 sysconf.load(confpath)
-                loaded = True
         self._confpath = confpath
 
         if os.path.isdir(datadir):
@@ -146,7 +144,7 @@ class Control(object):
                 os.makedirs(datadir)
                 writable = True
             except OSError:
-                raise Error, _("No configuration found!")
+                raise Error, _("Can't create datadir at %s") % datadir
 
         if writable and not self._pathlocks.lock(datadir, exclusive=True):
             writable = False
@@ -164,17 +162,20 @@ class Control(object):
                 return
 
             if self._cachechanged:
-                iface.showStatus(_("Saving cache..."))
                 cachepath = os.path.join(sysconf.get("data-dir"), "cache")
-                cachefile = open(cachepath+".new", "w")
-                state = (self.__stateversion__,
-                         self._cache,
-                         self._channels,
-                         self._sysconfchannels)
-                cPickle.dump(state, cachefile, 2)
-                cachefile.close()
-                os.rename(cachepath+".new", cachepath)
-                iface.hideStatus()
+                if sysconf.get("disk-cache", True):
+                    iface.showStatus(_("Saving cache..."))
+                    cachefile = open(cachepath+".new", "w")
+                    state = (self.__stateversion__,
+                             self._cache,
+                             self._channels,
+                             self._sysconfchannels)
+                    cPickle.dump(state, cachefile, 2)
+                    cachefile.close()
+                    os.rename(cachepath+".new", cachepath)
+                    iface.hideStatus()
+                elif os.path.isfile(cachepath):
+                    os.unlink(cachepath)
 
             if not sysconf.getModified():
                 return
@@ -214,7 +215,7 @@ class Control(object):
 
         if channels and not self._channels:
             cachepath = os.path.join(sysconf.get("data-dir"), "cache")
-            if os.path.isfile(cachepath):
+            if os.path.isfile(cachepath) and sysconf.get("disk-cache", True):
                 iface.showStatus(_("Loading cache..."))
                 cachefile = open(cachepath)
                 try:
@@ -290,6 +291,8 @@ class Control(object):
 
         # Get channels directory and check the necessary locks.
         channelsdir = os.path.join(sysconf.get("data-dir"), "channels/")
+        userchannelsdir = os.path.join(sysconf.get("user-data-dir"),
+                                       "channels/")
         if not os.path.isdir(channelsdir):
             try:
                 os.makedirs(channelsdir)
@@ -380,13 +383,14 @@ class Control(object):
                     pkgconf.setFlag("new", pkg.name, "=", pkg.version)
 
         # Remove unused files from channels directory.
-        if os.access(channelsdir, os.W_OK):
-            aliases = self._channels.copy()
-            aliases.update(dict.fromkeys(sysconf.get("channels", ()), True))
-            for entry in os.listdir(channelsdir):
-                sep = entry.find("%%")
-                if sep == -1 or entry[:sep] not in aliases:
-                    os.unlink(os.path.join(channelsdir, entry))
+        for dir in (channelsdir, userchannelsdir):
+            if os.access(dir, os.W_OK):
+                aliases = self._channels.copy()
+                aliases.update(dict.fromkeys(sysconf.get("channels", ())))
+                for entry in os.listdir(dir):
+                    sep = entry.find("%%")
+                    if sep == -1 or entry[:sep] not in aliases:
+                        os.unlink(os.path.join(dir, entry))
 
         # Change back to a shared lock.
         self._pathlocks.lock(channelsdir)
@@ -616,13 +620,54 @@ class Control(object):
         failed = fetcher.getFailedSet()
         if failed:
             raise Error, _("Failed to download packages:\n") + \
-                         "\n".join(["    %s: %s" % (url, failed[url])
+                         "\n".join([u"    %s: %s" % (url, failed[url])
                                     for url in failed])
         pkgpaths = {}
         for pkg in packages:
             pkgpaths[pkg] = [item.getTargetPath() for item in pkgitems[pkg]]
         return pkgpaths
 
+    def search(self, s, cutoff=1.00, suggestioncutoff=0.70,
+               globcutoff=1.00, globsuggestioncutoff=0.95,
+               addprovides=True):
+        ratio = 0
+        results = []
+        suggestions = []
+
+        objects = []
+
+        # If we find packages with exactly the given
+        # name or name-version, use them.
+        for pkg in self._cache.getPackages(s):
+            if pkg.name == s or "%s-%s" % (pkg.name, pkg.version) == s:
+                objects.append((1.0, pkg))
+         
+        if not objects:
+            if "*" in s:
+                cutoff = globcutoff
+                suggestioncutoff = globsuggestioncutoff
+            searcher = Searcher()
+            searcher.addAuto(s, suggestioncutoff)
+            if addprovides:
+                searcher.addProvides(s, suggestioncutoff)
+            self._cache.search(searcher)
+            objects = searcher.getResults()
+
+        if objects:
+            bestratio = objects[0][0]
+            if bestratio < cutoff:
+                suggestions = objects
+            else:
+                for i in range(len(objects)):
+                    ratio, obj = objects[i]
+                    if ratio == bestratio:
+                        results.append(obj)
+                    else:
+                        suggestions = objects[i:]
+                        break
+                if results:
+                    ratio = bestratio
+        return ratio, results, suggestions
 
 class AvailableChannelSet(object):
 
