@@ -20,7 +20,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 from smart.transaction import INSTALL, REMOVE, UPGRADE, REINSTALL, KEEP, FIX
-from smart.transaction import Transaction, ChangeSet, checkPackages
+from smart.transaction import Transaction, ChangeSet, checkPackagesSimple
 from smart.transaction import PolicyInstall, PolicyRemove, PolicyUpgrade
 from smart.interfaces.gtk.channels import GtkChannels, GtkChannelSelector
 from smart.interfaces.gtk.mirrors import GtkMirrors
@@ -31,6 +31,8 @@ from smart.interfaces.gtk.packageinfo import GtkPackageInfo
 from smart.interfaces.gtk.interface import GtkInterface
 from smart.interfaces.gtk import getPixbuf
 from smart.const import NEVER, VERSION
+from smart.searcher import Searcher
+from smart.cache import Package
 from smart import *
 import shlex, re
 import fnmatch
@@ -182,7 +184,10 @@ class GtkInteractiveInterface(GtkInterface):
         self._window.set_title("Smart Package Manager %s" % VERSION)
         self._window.set_position(gtk.WIN_POS_CENTER)
         self._window.set_geometry_hints(min_width=640, min_height=480)
-        self._window.connect("destroy", lambda x: gtk.main_quit())
+        def delete(widget, event):
+            gtk.main_quit()
+            return True
+        self._window.connect("delete-event", delete)
 
         self._log.set_transient_for(self._window)
         self._progress.set_transient_for(self._window)
@@ -306,7 +311,7 @@ class GtkInteractiveInterface(GtkInterface):
         hbox.show()
         searchtable.attach(hbox, 1, 2, 1, 2)
 
-        self._searchname = gtk.RadioButton(None, _("Name"))
+        self._searchname = gtk.RadioButton(None, _("Automatic"))
         self._searchname.set_active(True)
         self._searchname.connect("clicked", lambda x: self.refreshPackages())
         self._searchname.show()
@@ -315,10 +320,6 @@ class GtkInteractiveInterface(GtkInterface):
         self._searchdesc.connect("clicked", lambda x: self.refreshPackages())
         self._searchdesc.show()
         hbox.pack_start(self._searchdesc, False)
-        self._searchpath = gtk.RadioButton(self._searchname, _("Content"))
-        self._searchpath.connect("clicked", lambda x: self.refreshPackages())
-        self._searchpath.show()
-        hbox.pack_start(self._searchpath, False)
 
         # Packages and information
 
@@ -356,6 +357,7 @@ class GtkInteractiveInterface(GtkInterface):
 
     def run(self, command=None, argv=None):
         self.setCatchExceptions(True)
+        self.loadState()
         self._window.set_icon(getPixbuf("smart"))
         self._window.show()
         self._ctrl.reloadChannels()
@@ -364,10 +366,27 @@ class GtkInteractiveInterface(GtkInterface):
         self._progress.hide()
         self.refreshPackages()
         gtk.main()
+        self.saveState()
         self.setCatchExceptions(False)
 
     # Non-standard interface methods:
 
+    def saveState(self):
+        sysconf.set("gtk-size", self._window.get_size())
+        #sysconf.set("gtk-position", self._window.get_position())
+        sysconf.set("gtk-vpaned-position", self._vpaned.get_position())
+
+    def loadState(self):
+        var = sysconf.get("gtk-size")
+        if var is not None:
+            self._window.set_size_request(*var)
+        #var = sysconf.get("gtk-position")
+        #if var is not None:
+        #    self._window.move(*var)
+        var = sysconf.get("gtk-vpaned-position")
+        if var is not None:
+            self._vpaned.set_position(var)
+
     def getChangeSet(self):
         return self._changeset
 
@@ -389,10 +408,10 @@ class GtkInteractiveInterface(GtkInterface):
         self._changeset.setPersistentState(state)
         self.refreshPackages()
 
-    def applyChanges(self):
+    def applyChanges(self, confirm=True):
         transaction = Transaction(self._ctrl.getCache(),
                                   changeset=self._changeset)
-        if self._ctrl.commitTransaction(transaction):
+        if self._ctrl.commitTransaction(transaction, confirm=confirm):
             del self._undo[:]
             del self._redo[:]
             self._redomenuitem.set_property("sensitive", False)
@@ -430,10 +449,11 @@ class GtkInteractiveInterface(GtkInterface):
         if changeset != self._changeset:
             if self.confirmChange(self._changeset, changeset):
                 self.saveUndo()
+                emptychangeset = not self._changeset
                 self._changeset.setState(changeset)
                 self.changedMarks()
                 if self.askYesNo(_("Apply marked changes now?"), True):
-                    self.applyChanges()
+                    self.applyChanges(confirm=not emptychangeset)
         else:
             self.showStatus(_("No interesting upgrades available!"))
 
@@ -631,9 +651,11 @@ class GtkInteractiveInterface(GtkInterface):
         menu.popup(None, None, None, event.button, event.time)
 
     def checkPackages(self, all=False, uninstalled=False):
-        cache = self._ctrl.getCache()
-        if checkPackages(cache, cache.getPackages(), report=True,
-                         all=all, uninstalled=uninstalled):
+        installed = not uninstalled
+        available = all or uninstalled
+        self.info(_("Checking relations..."))
+        if checkPackagesSimple(self._ctrl.getCache(), report=True,
+                               installed=installed, available=available):
             self.info(_("All checked packages have correct relations."))
 
     def fixAllProblems(self):
@@ -716,10 +738,43 @@ class GtkInteractiveInterface(GtkInterface):
 
         tree = sysconf.get("package-tree", "groups")
         ctrl = self._ctrl
-        packages = ctrl.getCache().getPackages()
-
         changeset = self._changeset
 
+        if self._searchbar.get_property("visible"):
+
+            searcher = Searcher()
+            dosearch = False
+            if self._searchdesc.get_active():
+                text = self._searchentry.get_text().strip()
+                if text:
+                    dosearch = True
+                    searcher.addDescription(text)
+                    searcher.addSummary(text)
+            else:
+                try:
+                    tokens = shlex.split(self._searchentry.get_text())
+                except ValueError:
+                    pass
+                else:
+                    if tokens:
+                        dosearch = True
+                        for tok in tokens:
+                            if searcher.hasAutoMeaning(tok):
+                                searcher.addAuto(tok)
+                            else:
+                                searcher.addNameVersion("*%s*" % tok)
+
+            packages = []
+            if dosearch:
+                self._ctrl.getCache().search(searcher)
+                for ratio, obj in searcher.getResults():
+                    if isinstance(obj, Package):
+                        packages.append(obj)
+                    else:
+                        packages.extend(obj.packages)
+        else:
+            packages = ctrl.getCache().getPackages()
+
         filters = self._filters
         if filters:
             if "hide-non-upgrades" in filters:
@@ -748,39 +803,6 @@ class GtkInteractiveInterface(GtkInterface):
             if "hide-old" in filters:
                 packages = pkgconf.filterByFlag("new", packages)
 
-        if self._searchbar.get_property("visible"):
-            search = []
-            for tok in shlex.split(self._searchentry.get_text()):
-                tok = fnmatch.translate(tok)[:-1].replace(r"\ ", " ")
-                tok = r"\s+".join(tok.split())
-                search.append(re.compile(tok))
-            if not search:
-                packages = []
-            else:
-                desc = self._searchdesc.get_active()
-                path = self._searchpath.get_active()
-                newpackages = []
-                for pkg in packages:
-                    for pat in search:
-                        if desc:
-                            info = pkg.loaders.keys()[0].getInfo(pkg)
-                            if pat.search(info.getDescription()):
-                                newpackages.append(pkg)
-                                break
-                        elif path:
-                            info = pkg.loaders.keys()[0].getInfo(pkg)
-                            for path in info.getPathList():
-                                if pat.search(path):
-                                   newpackages.append(pkg)
-                                   break
-                            else:
-                                continue
-                            break
-                        elif pat.search(pkg.name):
-                            newpackages.append(pkg)
-                            break
-                packages = newpackages
-
         if tree == "groups":
             groups = {}
             done = {}
