@@ -20,7 +20,6 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 from smart.interfaces.text.interface import TextInterface, getScreenWidth
-from smart.matcher import MasterMatcher
 from smart.const import VERSION, NEVER
 from smart.option import OptionParser
 from smart.transaction import *
@@ -29,6 +28,7 @@ from cmd import Cmd
 import sys, os
 import shlex
 
+HISTFILE = "~/.smart_history"
 
 class TextInteractiveInterface(TextInterface):
 
@@ -36,7 +36,20 @@ class TextInteractiveInterface(TextInterface):
         print "Smart Package Manager %s - Shell Mode" % VERSION
         print
         self._ctrl.reloadChannels()
+        histfile = os.path.expanduser(HISTFILE)
+        try:
+            import readline
+            if os.path.isfile(histfile) and os.access(histfile, os.R_OK):
+                readline.read_history_file(histfile)
+        except ImportError:
+            readline = None
         Interpreter(self._ctrl).cmdloop()
+        if readline:
+            readline.set_history_length(100)
+            try:
+                readline.write_history_file(histfile)
+            except IOError:
+                pass
 
     def confirmChange(self, oldchangeset, newchangeset, expected=0):
         if newchangeset == oldchangeset:
@@ -69,6 +82,20 @@ class Interpreter(Cmd):
         self._undo = []
         self._redo = []
 
+        if sysconf.has("explain-changesets", soft=True):
+            self._explain = bool(sysconf.get("explain-changesets", soft=True))
+        else:
+            self._explain = None
+
+    def setExplain(self, value):
+        if value:
+            sysconf.set("explain-changesets", True, soft=True)
+        else:
+            if self._explain is None:
+                sysconf.remove("explain-changesets", soft=True)
+            else:
+                sysconf.set("explain-changesets", self._explain, soft=True)
+
     def completeAll(self, text, line, begidx, endidx):
         matches = []
         for pkg in self._ctrl.getCache().getPackages():
@@ -111,13 +138,33 @@ class Interpreter(Cmd):
             del self._redo[:]
             del self._undo[20:]
 
-    def pkgsFromLine(self, line):
-        args = shlex.split(line)
-        for arg in args:
-            matcher = MasterMatcher(arg)
-            pkgs = matcher.filter(self._ctrl.getCache().getPackages())
-            if not pkgs:
-                raise Error, _("'%s' matches no packages") % arg
+    def pkgsFromArgs(self, obj):
+        if type(obj) is str:
+            obj = shlex.split(obj)
+        for arg in obj:
+            ratio, results, suggestions = self._ctrl.search(arg,
+                                                            addprovides=False)
+            if not results:
+                if suggestions:
+                    dct = {}
+                    for r, obj in suggestions:
+                        if isinstance(obj, Package):
+                            dct[obj] = True
+                        else:
+                            dct.update(dct.fromkeys(obj.packages, True))
+                    raise Error, _("'%s' matches no packages. "
+                                   "Suggestions:\n%s") % \
+                                 (arg, "\n".join(["    "+str(x) for x in dct]))
+                else:
+                    raise Error, _("'%s' matches no packages") % arg
+            else:
+                pkgs = {}
+                for obj in results:
+                    if isinstance(obj, Package):
+                        pkgs[obj] = True
+                    else:
+                        pkgs.update(dict.fromkeys(obj.packages, True))
+            pkgs = pkgs.keys()
             if len(pkgs) > 1:
                 sortUpgrades(pkgs)
             yield arg, pkgs
@@ -153,6 +200,17 @@ class Interpreter(Cmd):
             sys.stderr.write(_("\nInterrupted\n"))
             return None
 
+    def print_topics(self, header, cmds, cmdlen, maxcol):
+        if cmds:
+            # The header may be a unicode string. That's the only
+            # reason to implement our own print_topics()
+            self.stdout.write(header)
+            self.stdout.write("\n")
+            if self.ruler:
+                self.stdout.write("%s\n"%str(self.ruler * len(header)))
+            self.columnize(cmds, maxcol-1)
+            self.stdout.write("\n")
+
     def help_help(self):
         print _("What would you expect!? ;-)")
 
@@ -194,16 +252,24 @@ class Interpreter(Cmd):
     def help_install(self):
         print _("The install command marks packages for installation.")
         print
+        print _("Options:")
+        print _("   --explain  Include additional information about\n"
+                "              changes, when possible")
+        print
         print _("Usage: install <pkgname> ...")
 
     complete_install = completeAvailable
     def do_install(self, line):
+        args = shlex.split(line)
+        parser = OptionParser(add_help_option=False)
+        parser.add_option("--explain", action="store_true")
+        opts, args = parser.parse_args(args)
         cache = self._ctrl.getCache()
         transaction = Transaction(cache, policy=PolicyInstall)
         transaction.setState(self._changeset)
         changeset = transaction.getChangeSet()
         expected = 0
-        for arg, pkgs in self.pkgsFromLine(line):
+        for arg, pkgs in self.pkgsFromArgs(args):
             expected += 1
             names = {}
             found = False
@@ -219,23 +285,35 @@ class Interpreter(Cmd):
             if not found:
                 raise Error, _("No uninstalled packages matched '%s'") % arg
         transaction.run()
+        if opts.explain:
+            self.setExplain(True)
         if iface.confirmChange(self._changeset, changeset, expected):
             self.saveUndo()
             self._changeset.setState(changeset)
+        if opts.explain:
+            self.setExplain(False)
 
     def help_reinstall(self):
         print _("The reinstall command marks packages for reinstallation.")
         print
+        print _("Options:")
+        print _("   --explain  Include additional information about\n"
+                "              changes, when possible")
+        print
         print _("Usage: reinstall <pkgname> ...")
 
     complete_reinstall = completeInstalled
     def do_reinstall(self, line):
+        args = shlex.split(line)
+        parser = OptionParser(add_help_option=False)
+        parser.add_option("--explain", action="store_true")
+        opts, args = parser.parse_args(args)
         cache = self._ctrl.getCache()
         transaction = Transaction(cache, policy=PolicyInstall)
         transaction.setState(self._changeset)
         changeset = transaction.getChangeSet()
         expected = 0
-        for arg, pkgs in self.pkgsFromLine(line):
+        for arg, pkgs in self.pkgsFromArgs(args):
             expected += 1
             if not pkgs:
                 raise Error, _("'%s' matches no installed packages") % arg
@@ -243,28 +321,40 @@ class Interpreter(Cmd):
                 raise Error, _("'%s' matches multiple installed packages")%arg
             transaction.enqueue(pkgs[0], REINSTALL)
         transaction.run()
+        if opts.explain:
+            self.setExplain(True)
         if iface.confirmChange(self._changeset, changeset, expected):
             self.saveUndo()
             self._changeset.setState(changeset)
+        if opts.explain:
+            self.setExplain(False)
 
     def help_upgrade(self):
         print _("The upgrade command marks packages for upgrading.")
         print
+        print _("Options:")
+        print _("   --explain  Include additional information about\n"
+                "              changes, when possible")
+        print
         print _("Usage: upgrade <pkgname> ...")
 
     complete_upgrade = completeInstalled
     def do_upgrade(self, line):
+        args = shlex.split(line)
+        parser = OptionParser(add_help_option=False)
+        parser.add_option("--explain", action="store_true")
+        opts, args = parser.parse_args(args)
         cache = self._ctrl.getCache()
         transaction = Transaction(cache, policy=PolicyUpgrade)
         transaction.setState(self._changeset)
         changeset = transaction.getChangeSet()
         expected = 0
-        if not line.strip():
+        if not args:
             for pkg in cache.getPackages():
                 if pkg.installed:
                     transaction.enqueue(pkg, UPGRADE)
         else:
-            for arg, pkgs in self.pkgsFromLine(line):
+            for arg, pkgs in self.pkgsFromArgs(args):
                 expected += 1
                 found = False
                 for pkg in pkgs:
@@ -274,26 +364,38 @@ class Interpreter(Cmd):
                 if not found:
                     raise Error, _("'%s' matches no installed packages") % arg
         transaction.run()
+        if opts.explain:
+            self.setExplain(True)
         if changeset == self._changeset:
             print _("No interesting upgrades available!")
         elif iface.confirmChange(self._changeset, changeset, expected):
             self.saveUndo()
             self._changeset.setState(changeset)
+        if opts.explain:
+            self.setExplain(False)
 
     def help_remove(self):
         print _("The remove command marks packages for being removed.")
         print
+        print _("Options:")
+        print _("   --explain  Include additional information about\n"
+                "              changes, when possible")
+        print
         print _("Usage: remove <pkgname> ...")
 
     complete_remove = completeInstalled
     def do_remove(self, line):
+        args = shlex.split(line)
+        parser = OptionParser(add_help_option=False)
+        parser.add_option("--explain", action="store_true")
+        opts, args = parser.parse_args(args)
         cache = self._ctrl.getCache()
         transaction = Transaction(cache, policy=PolicyRemove)
         transaction.setState(self._changeset)
         changeset = transaction.getChangeSet()
         policy = transaction.getPolicy()
         expected = 0
-        for arg, pkgs in self.pkgsFromLine(line):
+        for arg, pkgs in self.pkgsFromArgs(args):
             expected += 1
             found = False
             for pkg in pkgs:
@@ -306,23 +408,35 @@ class Interpreter(Cmd):
             if not found:
                 raise Error, _("'%s' matches no installed packages") % arg
         transaction.run()
+        if opts.explain:
+            self.setExplain(True)
         if iface.confirmChange(self._changeset, changeset, expected):
             self.saveUndo()
             self._changeset.setState(changeset)
+        if opts.explain:
+            self.setExplain(False)
 
     def help_keep(self):
         print _("The keep command unmarks currently marked packages.")
         print
+        print _("Options:")
+        print _("   --explain  Include additional information about\n"
+                "              changes, when possible")
+        print
         print _("Usage: keep <pkgname> ...")
 
     complete_keep = completeMarked
     def do_keep(self, line):
+        args = shlex.split(line)
+        parser = OptionParser(add_help_option=False)
+        parser.add_option("--explain", action="store_true")
+        opts, args = parser.parse_args(args)
         cache = self._ctrl.getCache()
         transaction = Transaction(cache, policy=PolicyInstall)
         transaction.setState(self._changeset)
         changeset = transaction.getChangeSet()
         expected = 0
-        for arg, pkgs in self.pkgsFromLine(line):
+        for arg, pkgs in self.pkgsFromArgs(args):
             expected += 1
             pkgs = [x for x in pkgs if x in changeset]
             if not pkgs:
@@ -330,33 +444,49 @@ class Interpreter(Cmd):
             for pkg in pkgs:
                 transaction.enqueue(pkg, KEEP)
         transaction.run()
+        if opts.explain:
+            self.setExplain(True)
         if iface.confirmChange(self._changeset, changeset, expected):
             self.saveUndo()
             self._changeset.setState(changeset)
+        if opts.explain:
+            self.setExplain(False)
 
     def help_fix(self):
         print _("The fix command verifies relations of given packages\n"
                 "and marks the necessary changes for fixing them.")
         print
+        print _("Options:")
+        print _("   --explain  Include additional information about\n"
+                "              changes, when possible")
+        print
         print _("Usage: fix <pkgname> ...")
 
     complete_fix = completeAll
     def do_fix(self, line):
+        args = shlex.split(line)
+        parser = OptionParser(add_help_option=False)
+        parser.add_option("--explain", action="store_true")
+        opts, args = parser.parse_args(args)
         cache = self._ctrl.getCache()
         transaction = Transaction(cache, policy=PolicyInstall)
         transaction.setState(self._changeset)
         changeset = transaction.getChangeSet()
         expected = 0
-        for arg, pkgs in self.pkgsFromLine(line):
+        for arg, pkgs in self.pkgsFromArgs(args):
             expected += 1
             for pkg in pkgs:
                 transaction.enqueue(pkg, FIX)
         transaction.run()
+        if opts.explain:
+            self.setExplain(True)
         if changeset == self._changeset:
             print _("No problems to resolve!")
         elif iface.confirmChange(self._changeset, changeset, expected):
             self.saveUndo()
             self._changeset.setState(changeset)
+        if opts.explain:
+            self.setExplain(False)
 
     def help_download(self):
         print _("The download command fetches the given packages\n"
@@ -367,7 +497,7 @@ class Interpreter(Cmd):
     complete_download = completeAll
     def do_download(self, line):
         packages = []
-        for arg, pkgs in self.pkgsFromLine(line):
+        for arg, pkgs in self.pkgsFromArgs(line):
             if len(pkgs) > 1:
                 iface.warning(_("'%s' matches multiple packages, "
                                 "selecting: %s") % (arg, pkgs[0]))
@@ -433,7 +563,6 @@ class Interpreter(Cmd):
 
     complete_ls = completeAll
     def do_ls(self, line):
-        pkgs = self._ctrl.getCache().getPackages()
         args = shlex.split(line)
         parser = OptionParser(add_help_option=False)
         parser.add_option("-i", action="store_true", dest="installed")
@@ -441,19 +570,37 @@ class Interpreter(Cmd):
         parser.add_option("-s", action="store_true", dest="summary")
         parser.add_option("-n", action="store_true", dest="new")
         opts, args = parser.parse_args(args)
+        if args:
+            pkgs = {}
+            for arg in args:
+                ratio, results, suggestions = self._ctrl.search(arg,
+                                                         addprovides=False)
+                if not results:
+                    if suggestions:
+                        dct = {}
+                        for r, obj in suggestions:
+                            if isinstance(obj, Package):
+                                dct[obj] = True
+                            else:
+                                dct.update(dct.fromkeys(obj.packages, True))
+                        raise Error, _("'%s' matches no packages. "
+                                       "Suggestions:\n%s") % \
+                                     (arg, "\n".join(["    "+str(x)
+                                                      for x in dct]))
+                    else:
+                        raise Error, _("'%s' matches no packages") % arg
+                else:
+                    for obj in results:
+                        if isinstance(obj, Package):
+                            pkgs[obj] = True
+                        else:
+                            pkgs.update(dict.fromkeys(obj.packages, True))
+        else:
+            pkgs = self._ctrl.getCache().getPackages()
         if opts.installed:
             pkgs = [x for x in pkgs if x.installed]
         if opts.new:
             pkgs = pkgconf.filterByFlag("new", pkgs)
-        if args:
-            newpkgs = []
-            for arg in args:
-                matcher = MasterMatcher(arg)
-                fpkgs = matcher.filter(pkgs)
-                if not fpkgs:
-                    raise Error, _("'%s' matches no packages") % arg
-                newpkgs.extend(fpkgs)
-            pkgs = newpkgs
         pkgs = dict.fromkeys(pkgs).keys()
         pkgs.sort()
 
@@ -551,7 +698,10 @@ class Interpreter(Cmd):
     def do_flag(self, line):
         from smart.commands import flag
         try:
-            opts = flag.parse_options(shlex.split(line))
+            try:
+                opts = flag.parse_options(shlex.split(line))
+            except ValueError, e:
+                raise Error, str(e)
             flag.main(self._ctrl, opts)
         except SystemExit:
             pass
@@ -567,7 +717,10 @@ class Interpreter(Cmd):
     def do_query(self, line):
         from smart.commands import query
         try:
-            opts = query.parse_options(shlex.split(line))
+            try:
+                opts = query.parse_options(shlex.split(line))
+            except ValueError, e:
+                raise Error, str(e)
             query.main(self._ctrl, opts, reloadchannels=False)
         except SystemExit:
             pass
@@ -581,7 +734,10 @@ class Interpreter(Cmd):
     def do_search(self, line):
         from smart.commands import search
         try:
-            opts = search.parse_options(shlex.split(line))
+            try:
+                opts = search.parse_options(shlex.split(line))
+            except ValueError, e:
+                raise Error, str(e)
             search.main(self._ctrl, opts, reloadchannels=False)
         except SystemExit:
             pass
@@ -595,7 +751,10 @@ class Interpreter(Cmd):
     def do_info(self, line):
         from smart.commands import info
         try:
-            opts = info.parse_options(shlex.split(line))
+            try:
+                opts = info.parse_options(shlex.split(line))
+            except ValueError, e:
+                raise Error, str(e)
             info.main(self._ctrl, opts, reloadchannels=False)
         except SystemExit:
             pass
@@ -608,10 +767,30 @@ class Interpreter(Cmd):
     def do_stats(self, line):
         from smart.commands import stats
         try:
-            opts = stats.parse_options(shlex.split(line))
+            try:
+                opts = stats.parse_options(shlex.split(line))
+            except ValueError, e:
+                raise Error, str(e)
             stats.main(self._ctrl, opts, reloadchannels=False)
         except SystemExit:
             pass
 
+    def help_check(self):
+        print _("The check command verifies relations between packages,\n"
+                "and accepts the same options available in the command\n"
+                "line interface.")
+        print
+        print _("Usage: check [options] [<pkgname>] ...")
+
+    def do_check(self, line):
+        from smart.commands import check
+        try:
+            try:
+                opts = check.parse_options(shlex.split(line))
+            except ValueError, e:
+                raise Error, str(e)
+            check.main(self._ctrl, opts, reloadchannels=False)
+        except SystemExit:
+            pass
 
 # vim:ts=4:sw=4:et
