@@ -253,7 +253,7 @@ class PolicyUpgrade(Policy):
         self._upgrading = upgrading = {}
         self._upgraded = upgraded = {}
         self._sortbonus = sortbonus = {}
-        self._requiredbonus = requiredbonus = {}
+        self._stablebonus = stablebonus = {}
         queue = self._trans.getQueue()
         for pkg in self._trans.getCache().getPackages():
             # Precompute upgrade relations.
@@ -289,6 +289,56 @@ class PolicyUpgrade(Policy):
                             else:
                                 upgraded[upgpkg] = [pkg]
 
+        # If package A-2.0 upgrades package A-1.0, and package A-2.0 is
+        # upgraded by A-3.0, give a bonus if A-1.0 is upgraded without
+        # installing A-2.0. In practice, this is a little bit more
+        # complex since there might be multiple levels of upgrades, and
+        # we can't penalize a possible package A2-2.0 which upgrades
+        # A-1.0 as well, and is not upgraded by anyone else.
+        for bonuspkg in upgraded:
+            upgmap = dict.fromkeys(upgraded[bonuspkg])
+            sb = {}
+            bonusvalue = {}
+            bonusdeps = {}
+            queue = [[bonuspkg]]
+            while queue:
+                path = queue.pop(0)
+                pathlen = len(path)
+                pkg = path[-1]
+                for prv in pkg.provides:
+                    for upg in prv.upgradedby:
+                        for upgpkg in upg.packages:
+                            if (not upgpkg.installed and
+                                upgpkg in upgmap and upgpkg not in path and
+                                self.getPriority(pkg) <=
+                                self.getPriority(upgpkg)):
+                                if pathlen > 1:
+                                    # Paths always increase in size, so we can
+                                    # be sure that the value being introduced
+                                    # here is >= to the previous one.
+                                    bonusvalue[pkg] = -30*(pathlen-1)
+                                    deps = bonusdeps.setdefault(pkg, {})
+                                    for pathpkg in path[1:]:
+                                        deps[pathpkg] = True
+                                queue.append(path+[upgpkg])
+                for upg in pkg.upgrades:
+                    for prv in upg.providedby:
+                        for prvpkg in prv.packages:
+                            if (not prvpkg.installed and
+                                prvpkg in upgmap and prvpkg not in path and
+                                self.getPriority(pkg) <
+                                self.getPriority(prvpkg)):
+                                if pathlen > 1:
+                                    bonusvalue[pkg] = -30*(pathlen-1)
+                                    deps = bonusdeps.setdefault(pkg, {})
+                                    for pathpkg in path[1:-1]:
+                                        deps[pathpkg] = True
+                                queue.append(path+[prvpkg])
+            if bonusvalue:
+                lst = [(bonusvalue[pkg], bonusdeps[pkg]) for pkg in bonusvalue]
+                lst.sort()
+                stablebonus[bonuspkg] = lst
+
         pkgs = self._trans._queue.keys()
         sortUpgrades(pkgs, self)
         for i, pkg in enumerate(pkgs):
@@ -304,7 +354,7 @@ class PolicyUpgrade(Policy):
         upgrading = self._upgrading
         upgraded = self._upgraded
         sortbonus = self._sortbonus
-        requiredbonus = self._requiredbonus
+        stablebonus = self._stablebonus
 
         installedcount = 0
         upgradedmap = {}
@@ -326,6 +376,18 @@ class PolicyUpgrade(Policy):
                 if upgpkgs:
                     weight += sortbonus.get(pkg, 0)
                     upgradedmap.update(upgpkgs)
+
+        for pkg in upgradedmap:
+            sb = stablebonus.get(pkg)
+            if sb:
+                for bonusvalue, bonusdeps in sb:
+                    for deppkg in bonusdeps:
+                        if deppkg in changeset:
+                            break
+                    else:
+                        weight += bonusvalue
+                        break
+
         upgradedcount = len(upgradedmap)
         weight += -30*upgradedcount+(installedcount-upgradedcount)
         return weight
@@ -809,7 +871,6 @@ class Transaction(object):
         getweight = self._policy.getWeight
 
         sortUpgrades(pkgs, self._policy)
-        pkgs.reverse()
 
         lockedstate = {}
 
@@ -1617,45 +1678,50 @@ def getAlternates(pkg, cache):
 
     return alternates
 
-def checkPackages(cache, pkgs, report=False, all=False, uninstalled=False):
-    pkgs.sort()
+def checkPackagesSimple(cache, checkset=None, report=False,
+                        installed=False, available=False):
+    if installed and available:
+        relateset = cache.getPackages()
+    elif installed:
+        relateset = [pkg for pkg in cache.getPackages() if pkg.installed]
+    elif available:
+        relateset = []
+        for pkg in cache.getPackages():
+            if not pkg.installed:
+                relateset.append(pkg)
+            else:
+                for loader in pkg.loaders:
+                    if not loader.getInstalled():
+                        relateset.append(pkg)
+                        break
+    else:
+        raise Error, "checkPackagesSimple() called with invalid parameters"
+    if checkset is None:
+        checkset = relateset
+    return checkPackages(cache, checkset, relateset, report)
+
+def checkPackages(cache, checkset, relateset, report=False):
+    checkset = list(checkset)
+    checkset.sort()
+    relateset = dict.fromkeys(relateset, True)
 
     problems = False
     coexistchecked = {}
-    for pkg in pkgs:
-
-        if not all:
-            if uninstalled:
-                for loader in pkg.loaders:
-                    if not loader.getInstalled():
-                        break
-                else:
-                    continue
-            elif not pkg.installed:
-                continue
-
+    for pkg in checkset:
         for req in pkg.requires:
             for prv in req.providedby:
                 for prvpkg in prv.packages:
-                    if all:
-                        break
-                    elif uninstalled:
-                        for loader in prvpkg.loaders:
-                            if not loader.getInstalled():
-                                break
-                        else:
-                            continue
-                        break
-                    elif prvpkg.installed:
+                    if prvpkg in relateset:
                         break
                 else:
                     continue
                 break
             else:
-                if report:
-                    iface.info(_("Unsatisfied dependency: %s requires %s") %
-                               (pkg, req))
+                if not report:
+                    return False
                 problems = True
+                iface.info(_("Unsatisfied dependency: %s requires %s") %
+                           (pkg, req))
 
         if not pkg.installed:
             continue
@@ -1663,27 +1729,45 @@ def checkPackages(cache, pkgs, report=False, all=False, uninstalled=False):
         for cnf in pkg.conflicts:
             for prv in cnf.providedby:
                 for prvpkg in prv.packages:
-                    if prvpkg is pkg:
-                        continue
-                    if prvpkg.installed:
-                        if report:
-                            iface.info(_("Unsatisfied dependency: "
-                                         "%s conflicts with %s") %
-                                       (pkg, prvpkg))
+                    if (prvpkg is not pkg and
+                        prvpkg.installed and
+                        prvpkg in relateset):
+                        if not report:
+                            return False
                         problems = True
+                        iface.info(_("Unsatisfied dependency: "
+                                     "%s conflicts with %s") % (pkg, prvpkg))
 
         namepkgs = cache.getPackages(pkg.name)
         for namepkg in namepkgs:
-            if (namepkg, pkg) in coexistchecked:
-                continue
-            coexistchecked[(pkg, namepkg)] = True
-            if (namepkg.installed and namepkg is not pkg and
-                not pkg.coexists(namepkg)):
-                if report:
+            if (namepkg is not pkg and
+                namepkg.installed and
+                namepkg in relateset and
+                (namepkg, pkg) not in coexistchecked):
+                coexistchecked[(pkg, namepkg)] = True
+                if not pkg.coexists(namepkg):
+                    if not report:
+                        return False
+                    problems = True
                     iface.info(_("Package %s can't coexist with %s") %
                                (namepkg, pkg))
-                problems = True
 
     return not problems
 
+def enablePsyco(psyco):
+    psyco.bind(PolicyInstall.getWeight)
+    psyco.bind(PolicyRemove.getWeight)
+    psyco.bind(PolicyUpgrade.getWeight)
+    psyco.bind(Transaction._install)
+    psyco.bind(Transaction._remove)
+    psyco.bind(Transaction._updown)
+    psyco.bind(Transaction._pending)
+    psyco.bind(Transaction._upgrade)
+    psyco.bind(Transaction.enqueue)
+    psyco.bind(sortUpgrades)
+    psyco.bind(recursiveUpgrades)
+    psyco.bind(checkPackages)
+
+hooks.register("enable-psyco", enablePsyco)
+
 # vim:ts=4:sw=4:et
